# -*- coding: utf-8 -*-
import re
import sys

from suite_py.lib.handler.youtrack_handler import YoutrackHandler
from suite_py.lib.handler.github_handler import GithubHandler
from suite_py.lib.handler import git_handler as git
from suite_py.lib.config import Config
from suite_py.lib.logger import Logger
from suite_py.lib.handler import prompt_utils


youtrack = YoutrackHandler()
github = GithubHandler()
logger = Logger()
config = Config()


def entrypoint(project, card):
    if git.is_dirty(project):
        logger.error("Hai modifiche non committate, non posso continuare")
        sys.exit(-1)

    try:
        if card:
            issue = youtrack.get_issue(card)
        else:
            issue = youtrack.get_issue(ask_card())
    except Exception:
        logger.error(
            "Si e' verificato un problema recuperando la issue da youtrack. Controlla che il numero della issue e' corretto"
        )
        sys.exit(-1)

    checkout_branch(project, issue)

    youtrack.assign_to(issue["id"], "me")

    youtrack.update_state(issue["id"], config.load()["youtrack"]["picked_state"])


def ask_card():
    return prompt_utils.ask_questions_input(
        "Inserisci il numero della issue youtrack:",
        config.load()["user"]["default_slug"],
    )


def checkout_branch(project, issue):
    branch_name = prompt_utils.ask_questions_input(
        "Inserisci nome del branch: ",
        re.sub(
            r'([\s\\.,~\^:\(\)\[\]\<\>"\'?]|[^\x00-\x7F])+', "-", issue["summary"]
        ).lower(),
    )

    default_parent_branch_name = git.current_branch_name(project)

    parent_branch_name = prompt_utils.ask_questions_input(
        "Inserisci branch iniziale: ", default_parent_branch_name
    )

    branch_type = issue["Type"].lower().replace(" ", "-")

    branch_name = f"{branch_type}/{branch_name}/{issue['id']}"

    git.checkout(project, parent_branch_name)

    git.checkout(project, branch_name)



# -*- coding: utf-8 -*-
import subprocess
import sys
import yaml

from halo import Halo

from suite_py.lib.handler.youtrack_handler import YoutrackHandler
from suite_py.lib.handler.github_handler import GithubHandler
from suite_py.lib.handler import git_handler as git
from suite_py.lib.handler import drone_handler as drone
from suite_py.lib.handler import prompt_utils
from suite_py.lib.logger import Logger
from suite_py.lib.qainit import get_qa_projects, qainit_deploy
from suite_py.lib.config import Config


youtrack = YoutrackHandler()
github = GithubHandler()
logger = Logger()
config = Config()


def entrypoint(project):
    qa_projects = get_qa_projects()
    if project not in qa_projects:
        logger.error(
            f"Il progetto {project} non fa parte degli ambienti di QA, niente da fare"
        )
        sys.exit(0)

    with Halo(text="Updating prima-twig...", spinner="dots", color="magenta"):
        try:
            subprocess.run(
                ["gem", "update", "prima-twig"],
                check=True,
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            )
        except Exception:
            logger.error(
                "Errore durante l'aggiornamento di prima-twig, per favore esegui manualmente `gem update prima-twig`"
            )
            sys.exit(-1)

    branch_name = git.current_branch_name(project)
    youtrack_id = youtrack.get_card_from_name(branch_name)

    if youtrack_id:
        if prompt_utils.ask_confirm(
            f"Ci sono altri progetti da aggiungere al QA che fanno riferimento alla card {youtrack_id}?",
            default=False,
        ):
            twig_arg = multi_repo_qa(youtrack_id, qa_projects)
        else:
            twig_arg = mono_repo_qa(project, branch_name)
    else:
        logger.warning(
            "Non sono riuscito a trovare una issue YouTrack collegata al branch, non e' possibile collegare automaticamente altri progetti a questo QA"
        )
        logger.warning(
            "Se vuoi creare il QA con diversi branch su diversi progetti per favore interrompi il comando ed utilizza twig"
        )
        twig_arg = mono_repo_qa(project, branch_name)

    qainit_deploy(twig_arg)

    if youtrack_id:
        logger.info("Aggiorno la card su youtrack...")
        youtrack.update_state(youtrack_id, config.load()["youtrack"]["test_state"])

    drone_url = drone.get_last_build_url("qainit", git.get_username())
    if drone_url:
        logger.info(f"Puoi seguire la creazione del QA su {drone_url}")

    logger.info("Configurazione del QA effettuata con successo!")


def multi_repo_qa(youtrack_id, qa_projects):
    logger.info("Cerco gli altri branch su github...")
    repos = list(map(github.get_repo, qa_projects))

    selected_repos = []
    for repo in repos:
        print(".", end="", flush=True)
        for branch in repo.get_branches():
            if youtrack_id in branch.name:
                selected_repos.append((repo.name, branch))

    print(".")
    logger.info("Progetti trovati:")
    for elem in selected_repos:
        logger.info(f"{elem[0]} --> {elem[1].name}")

    return twigify(selected_repos)


def mono_repo_qa(project, branch_name):
    logger.info(f"Creo il qa con il progetto {project} e branch {branch_name}")
    gh_branch = github.get_repo(project).get_branch(branch_name)
    selected_repos = [(project, gh_branch)]

    return twigify(selected_repos)


def twigify(projects_list):
    projects_dict = {}
    for project in projects_list:
        committer = ""

        try:
            committer = project[1].commit.committer.email or ""
        except Exception:
            committer = project[1].commit.commit.committer.email or ""

        projects_dict[project[0]] = {
            "name": project[1].name,
            "revision": project[1].commit.sha[:15],
            "committer": committer,
            "default_branch": False,
        }

    return yaml.dump(projects_dict, explicit_start=True, default_flow_style=False)



# -*- encoding: utf-8 -*-
from distutils.version import StrictVersion
from os import environ
import time
import subprocess
from subprocess import CalledProcessError
import sys
import requests

from halo import Halo
from suite_py.lib.tokens import Tokens
from suite_py.lib.logger import Logger

logger = Logger()
drone_token = environ["DRONE_TOKEN"] = Tokens().drone
baseurl = environ["DRONE_SERVER"] = "https://drone-1.prima.it"


def get_last_build_url(repo, prefix=None):
    with Halo(text="Contacting drone...", spinner="dots", color="magenta"):
        # necessario per far comparire la build che abbiamo appena pushato
        time.sleep(2)
        try:
            builds = requests.get(
                f"{baseurl}/api/repos/primait/{repo}/builds",
                headers={"Authorization": f"Bearer {drone_token}"},
            ).json()

            if prefix:
                builds = [b for b in builds if b["target"].startswith(prefix)]

            return f"{baseurl}/primait/{repo}/{builds[0]['number']}"
        except Exception:
            return ""


def get_pr_build_number(repo, commit_sha):
    with Halo(text="Contacting drone...", spinner="dots", color="magenta"):
        tries = 10
        while tries > 0:
            tries -= 1
            try:
                builds = requests.get(
                    f"{baseurl}/api/repos/primait/{repo}/builds?per_page=100",
                    headers={"Authorization": f"Bearer {drone_token}"},
                ).json()
                builds = [b for b in builds if b["after"] == commit_sha]
                return builds[0]["number"]
            except Exception:
                time.sleep(1)

        return None


def get_user():
    try:
        user = requests.get(
            f"{baseurl}/api/user", headers={"Authorization": f"Bearer {drone_token}"},
        ).json()
        return user
    except Exception:
        return None


def get_build_url(repo, build_number):
    if build_number:
        return f"{baseurl}/primait/{repo}/{build_number}"
    return None


def get_tags_from_builds(repo):
    tags = []
    builds = requests.get(
        f"{baseurl}/api/repos/primait/{repo}/builds?per_page=100",
        headers={"Authorization": f"Bearer {drone_token}"},
    ).json()

    for build in builds:
        if build["event"] == "tag":
            tags.append(build["ref"].replace("refs/tags/", ""))

    tags = list(dict.fromkeys(tags))
    tags.sort(key=StrictVersion, reverse=True)
    return tags


def get_build_from_tag(repo, tag):
    attempts = 0
    while attempts < 3:

        builds = requests.get(
            f"{baseurl}/api/repos/primait/{repo}/builds?per_page=100",
            headers={"Authorization": f"Bearer {drone_token}"},
        ).json()

        for build in builds:
            if build["event"] == "tag":
                if build["ref"].replace("refs/tags/", "") == tag:
                    return build["number"]

        time.sleep(2)
        attempts += 1

    return None


def launch_build(repo, build):
    return requests.post(
        f"{baseurl}/api/repos/primait/{repo}/builds/{build}",
        headers={"Authorization": f"Bearer {drone_token}"},
    ).json()


def prestart_success(repo, build_number):
    with Halo(text="Contacting drone...", spinner="dots", color="magenta"):
        tries = 10
        while build_number and tries > 0:
            tries -= 1
            build_status = requests.get(
                f"{baseurl}/api/repos/primait/{repo}/builds/{build_number}",
                headers={"Authorization": f"Bearer {drone_token}"},
            ).json()

            steps = build_status["stages"][0]["steps"]

            for step in steps:
                if step["name"] == "pre-start":
                    if step["status"] == "success":
                        return True
                    break
            time.sleep(3)
        return False


def fmt(project, path):
    try:
        subprocess.run(["drone", "fmt", "--save", ".drone.yml"], cwd=path, check=True)
    except (CalledProcessError) as e:
        logger.error(f"{project}: non sono riuscito a formattare il .drone.yml {e}")
        sys.exit(-1)


def validate(project, path):
    try:
        subprocess.run(
            ["drone", "lint", "--trusted", ".drone.yml"], cwd=path, check=True
        )
    except (CalledProcessError) as e:
        logger.error(f"{project}: il .drone.yml non è valido {e}")
        sys.exit(-1)


def sign(project, path):
    try:
        subprocess.run(
            ["drone", "sign", f"primait/{project}", "--save"], cwd=path, check=True,
        )
    except (CalledProcessError) as e:
        logger.error(f"{project}: non sono riuscito a firmare il .drone.yml {e}")
        sys.exit(-1)



# -*- encoding: utf-8 -*-
import subprocess
import yaml

from suite_py.lib.logger import Logger
from suite_py.lib.config import Config
from suite_py.lib.handler import git_handler as git
from suite_py.lib.handler.github_handler import GithubHandler


config = Config()
logger = Logger()
github = GithubHandler()

# development only
# twig_command = "{}/twig-binaries/bin/twig-feature".format(
#     config.load()["user"]["projects_home"]
# )

qainit_dir = f"{config.load()['user']['projects_home']}/qainit"


def get_qa_projects():
    git.check_repo_cloned("qainit")
    git.sync("qainit")
    with open(f"{qainit_dir}/projects.yml", "r") as file:
        branches_obj = yaml.load(file.read())

    return list(branches_obj.keys())


def qainit_deploy(args):
    with open(f"{qainit_dir}/suitepy-projects.yml", "w+") as file:
        file.write(args)

    return subprocess.run(
        # [twig_command, "suite", "deploy"], # development only
        ["twig", "feature", "suite", "deploy"],
        cwd=qainit_dir,
        check=True,
    )


def qainit_shutdown(youtrack_id):
    git.check_repo_cloned("qainit")
    git.sync("qainit")
    branch = git.search_remote_branch("qainit", f"*{youtrack_id}*")
    if branch:
        git.checkout("qainit", branch)
        git.commit("qainit", "shutdown", dummy=True)
        git.push("qainit", branch)

    else:
        logger.warning(
            "Non sono riuscito a trovare un qa per questa issue, se il qa esiste, per favore spegnilo manualmente\nDevops are watching you! ( •͡˘ _•͡˘)"
        )
